from gcpds.utils import loaddb
import unittest

import logging
logging.basicConfig(level=logging.DEBUG)


########################################################################
class TestLoadDB(unittest.TestCase):
    """"""

    # ----------------------------------------------------------------------
    def _test_database(self, database):
        """"""
        db = getattr(loaddb, database)(database)
        db.load_subject(1)

        run, classes = db.get_run(0)
        self.assertEqual(run.shape[0], classes.shape[0])

        run, classes = db.get_run(0, classes=[0, 1])
        self.assertEqual(len(set(classes)), 2)

        run, classes = db.get_run(0, channels=[1, 2, 3, 4])
        self.assertEqual(run.shape[1], 4)

        run, classes = db.get_run(0, classes=[0, 1], channels=[1, 2, 3])
        self.assertEqual(run.shape[1], 3)
        self.assertEqual(len(set(classes)), 2)

        self.assertRaises(Exception, lambda: db.get_run(
            0, classes=[0, 1], channels=[0, 2, 3]))

        self.assertRaises(Exception, lambda: db.load_subject(999))

        db.get_epochs(0)

    # ----------------------------------------------------------------------
    def test_bci2a(self):
        """"""
        self._test_database('BCI2a')

    # ----------------------------------------------------------------------
    def test_giga(self):
        """"""
        self._test_database('GIGA')

    # ----------------------------------------------------------------------
    def test_high_gamma(self):
        """"""
        # Could kill a low memory computer
        self._test_database('HighGamma')












from .dummy import Dumb
from typing import Union


# ----------------------------------------------------------------------
def dumber(n: Union[int, float], m: Union[int, float]) -> Union[int, float]:
    """Dumper way to add two numbers."""
    dumb = Dumb(n)
    return dumb.add_m(m)




""""""

from .google_drive_downloader import GoogleDriveDownloader as gdd
from scipy.io import loadmat
import os
from abc import ABCMeta, abstractmethod
from typing import Union, Optional, Tuple
import numpy as np
import json
import mne
import tables
import logging
import warnings

ALL = 'all'


# ----------------------------------------------------------------------
def load_mat(path: str, mat: str, fid: str, size: Optional[int] = None, overwrite: Optional[bool] = False) -> np.ndarray:
    """Get the raw data for one individual file.

    If the file does not exist in the specified path then tries to download it
    from Google Drive.
    """

    filepath = os.path.join(path, mat)

    if os.path.exists(filepath) and not overwrite:
        try:
            return loadmat(filepath)
        except ValueError:
            try:
                return tables.open_file(filepath, driver="H5FD_CORE")
            except tables.exceptions.HDF5ExtError:
                logging.warning('Corrupt database!!\n, overwriting...')
                return load_mat(path, mat, fid, size, overwrite=True)

        except:
            logging.warning('Corrupt database!!\n, overwriting...')
            return load_mat(path, mat, fid, size, overwrite=True)

    else:
        logging.warning('Database not found!')
        logging.warning('downloading...')

        os.makedirs(path, exist_ok=True)
        gdd.download_file_from_google_drive(file_id=fid,
                                            dest_path=filepath,
                                            unzip=False,
                                            overwrite=overwrite,
                                            size=size)
        return load_mat(path, mat, fid, size)


# ----------------------------------------------------------------------
def download_metadata(path, metadata):
    """"""
    os.makedirs(path, exist_ok=True)
    for file in metadata:
        fid, size = metadata[file]
        gdd.download_file_from_google_drive(file_id=fid,
                                            dest_path=os.path.join(path, file),
                                            unzip=False,
                                            overwrite=True,
                                            size=size)


# # ----------------------------------------------------------------------
# def get_menmap_filename():
    # """"""
    # filename = ''.join([random.choice(string.ascii_lowercase)
                        # for i in range(16)])
    # return f'{filename}.menmap'


########################################################################
class Database(metaclass=ABCMeta):
    """"""

    # ----------------------------------------------------------------------
    def __init__(self, path: Optional[str] = '.') -> None:
        """Constructor"""
        self.path = path
        # self.usemenmap = usemenmap

    # ----------------------------------------------------------------------
    @abstractmethod
    def load_subject(self, subject: int, mode: str) -> None:
        """"""
        if not mode in ['training', 'evaluation']:
            raise Exception(
                f"No mode {mode} available, only 'training', 'evaluation'")

        filename_subject = self.metadata[f'subject_{mode}_pattern'](subject)

        if os.path.split(filename_subject)[-1] not in self.metadata[f'subject_{mode}_files'].keys():
            raise Exception(f"Subject {subject} not in list of subjects.")

        fid, size = self.metadata[f'subject_{mode}_files'][os.path.split(
            filename_subject)[-1]]

        self.subject = subject
        self.mode = mode

        self.runs = self.metadata[f'runs_{mode}'][subject - 1]
        # self.data = load_mat(self.path, filename_subject, fid)['eeg'][0][0]
        return load_mat(self.path, filename_subject, fid, size)

    # # ----------------------------------------------------------------------
    # def to_menmap(self, array):
        # """"""
        # array = np.array(array.tolist())

        # filename = os.path.join(self.path, get_menmap_filename())
        # fp = np.memmap(filename, dtype=array.dtype,
                       # mode='w+', shape=array.shape)
        # fp[:] = array[:]
        # del array, fp
        # mmap = np.memmap(filename, mode='r')
        # return mmap

    # ----------------------------------------------------------------------
    @abstractmethod
    def get_run(self, run: int, classes: Union[int, str], channels=Union[int, str], reject_bad_trials: Optional[bool] = True) -> np.ndarray:
        """"""
        if run > self.runs:
            raise Exception(f'The current user only have {self.runs} runs.')

        if isinstance(channels, (list, tuple)) and -1 in channels:
            raise Exception('The channels are 1-based arrays')

        if isinstance(classes, (list, tuple)) and np.max(classes) >= len(self.metadata['classes']):
            raise Exception(
                f"The class index {np.max(classes)} is out of range.")

    # ----------------------------------------------------------------------
    def get_data(self, classes: Optional[list] = ALL, channels: Optional[list] = ALL, reject_bad_trials: Optional[bool] = True):
        """Return all runs."""

        start = 0
        for _ in range(self.runs):
            r, c = self.get_run(start, classes=classes, channels=channels,
                                reject_bad_trials=reject_bad_trials)
            if not r is None:
                break
            else:
                start += 1

        for run in range(start + 1, self.runs):
            r_, c_ = self.get_run(
                run, classes=classes, channels=channels, reject_bad_trials=reject_bad_trials)
            if not r_ is None:
                r = np.concatenate([r, r_], axis=0)
                c = np.concatenate([c, c_])

        return r, c

    # ----------------------------------------------------------------------
    def format_channels_selectors(self, channels=None):
        """Generate the channels vector.

        If no selector then all channels will be used, the channels cant be
        indicated with the name or the index. If index is used this must be
        1-based array.
        """

        if channels != ALL:
            channels = [self.metadata['channel_names'].index(
                ch) if isinstance(ch, str) else (ch - 1) for ch in channels]
        else:
            channels = list(range(len(self.metadata['channel_names'])))

        return channels

    # ----------------------------------------------------------------------
    def format_class_selector(self, classes):
        """"""
        if classes != ALL:
            classes = [self.metadata['classes'].index(
                cls) if isinstance(cls, str) else cls for cls in classes]
        else:
            classes = range(len(self.metadata['classes']))

        return classes

    # ----------------------------------------------------------------------
    def get_epochs(self, run=ALL, **kwargs):
        """"""
        # Remove channels that not correspond with the montage
        montage = mne.channels.make_standard_montage(self.metadata['montage'])
        channels_names = set(self.metadata['channel_names']).intersection(
            set(montage.ch_names))
        channels_missings = set(self.metadata['channel_names']).difference(
            set(montage.ch_names))

        logging.warning(
            f"Missing {channels_missings} channels in {self.metadata['montage']} montage.\n"
            f"Missing channels will be removed from MNE Epochs")

        info = mne.create_info(
            list(channels_names), sfreq=self.metadata['sampling_rate'], ch_types="eeg")
        info.set_montage(self.metadata['montage'])

        if run != ALL:
            data, classes = self.get_run(
                run, channels=list(channels_names))
        else:
            data, classes = self.get_data(
                run, channels=list(channels_names))

        events = [[i, 1, cls] for i, cls in enumerate(classes)]
        event_id = {e: i for i, e in enumerate(self.metadata['classes'])}

        return mne.EpochsArray(data, info, events=events, tmin=self.metadata['tmin'], event_id=event_id, **kwargs)

    # ----------------------------------------------------------------------
    def get_metadata(self):
        """"""
        download_metadata(os.path.join(self.path, 'metadata'),
                          self.metadata['metadata'])


########################################################################
class BCIilliteracy(Database):
    """"""

    # ----------------------------------------------------------------------
    def load_subject(self, subject: int, mode: str) -> None:
        """"""
        if not mode in ['training', 'evaluation']:
            raise Exception(
                f"No mode {mode} available, only 'training', 'evaluation'")

        self.runs = self.metadata[f'runs'][subject - 1]

        sessions = []
        for run in range(self.runs):
            filename_subject = self.metadata[f'subject_pattern'](
                subject, run + 1)

            if os.path.split(filename_subject)[-1] not in self.metadata[f'subject_files'].keys():
                raise Exception(f"Subject {subject} not in list of subjects.")

            fid, size = self.metadata[f'subject_files'][os.path.split(
                filename_subject)[-1]]

            self.subject = subject
            self.mode = mode

            sessions.append(load_mat(self.path, filename_subject, fid, size))

        return sessions

    # ----------------------------------------------------------------------
    def get_run(self, run: int, classes: Optional[list] = ALL, channels: Optional[list] = ALL, reject_bad_trials: Optional[bool] = True) -> Tuple[np.ndarray, np.ndarray]:
        """"""
        classes = self.format_class_selector(classes)
        channels = self.format_channels_selectors(channels)
        super().get_run(run, classes, channels, reject_bad_trials)

        data = self.data_[run]

        classes_list = data[4][0]
        starts = data[2][0]
        end = int(self.metadata['sampling_rate'] * self.metadata['duration'])

        run = np.array([data[1][start:start + end] for start in starts])

        # trial x channel x time
        run = np.moveaxis(run, 2, 1)

        # Select channels
        run = run[:, channels, :]

        idx = []
        c = []
        for cls in classes:
            idx.append(np.where(np.array(classes_list) == cls + 1)[0])
            c.append([cls] * len(idx[-1]))

        return run[np.concatenate(idx), :, :], np.concatenate(c)




import numpy as np
from scipy.fftpack import fft, fftfreq, fftshift
from scipy.signal import welch as sci_welch
from datetime import datetime


# ----------------------------------------------------------------------
def fourier(eeg, d=None, fs=None, timestamp=None, axis=-1):
    """Spectral power density using Fourier."""

    timestamp = np.array(timestamp)

    if timestamp.any():
        if isinstance(timestamp[-1], (int, float)):
            delta = datetime.fromtimestamp(
                timestamp[-1]) - datetime.fromtimestamp(timestamp[0])
        else:
            delta = timestamp[-1] - timestamp[0]
        fs = max(eeg.shape) / delta.total_seconds()

    if fs:
        d = 1 / fs

    Yf = fftshift(np.abs(fft(eeg, axis=axis)))
    f = fftshift(fftfreq(Yf.shape[1], d))

    n = (Yf.shape[1] // 2)
    Yf = Yf[:, n:] * (2 / Yf.shape[1])
    f = f[n:]

    return f, Yf


# ----------------------------------------------------------------------
def welch(eeg, d=None, fs=None, timestamp=None, axis=-1):
    """Spectral power density using Welch's method."""

    timestamp = np.array(timestamp)

    if timestamp.any():
        if isinstance(timestamp[-1], (int, float)):
            delta = datetime.fromtimestamp(
                timestamp[-1]) - datetime.fromtimestamp(timestamp[0])
        else:
            delta = timestamp[-1] - timestamp[0]
        fs = max(eeg.shape) / delta.total_seconds()

    if fs:
        d = 1 / fs

    # if axis is None:
        # # if eeg.shape[0] > eeg.shape[1]:
            # # axis = 0
            # # m_ = eeg.shape[0]
        # # else:
            # # axis = 1
            # # m_ = eeg.shape[1]
    # # else:
        # # m_ = eeg.shape[0]

    m_ = max(eeg.shape)

    f, p = sci_welch(eeg, 1 / d, window='flattop',
                     nperseg=min(m_, 1024), scaling='spectrum', axis=axis)
    return f, p





import numpy as np
import mne
import pandas as pd


# ----------------------------------------------------------------------
def apply_along_epochs(func1d, epochs):
    """"""
    epochs_ = epochs.copy()

    data = np.apply_along_axis(func1d, 2, epochs_)

    new_fs = data.shape[2] / \
        (epochs_.get_data().shape[2] / epochs_.info['sfreq'])
    epochs_.resample(new_fs, npad='auto')
    epochs_._data = data
    # epochs_._data = data - data.mean(axis=2)[..., np.newaxis]

    return epochs_


# ----------------------------------------------------------------------
def get_best_montage(channels):
    """"""
    data = {}
    target = set(channels)

    for montage_name in mne.channels.get_builtin_montages():
        montage = mne.channels.make_standard_montage(montage_name)
        count = len(target.intersection(set(montage.ch_names)))
        missings = target.difference(set(montage.ch_names))
    #     counts.append([count, missings, montage_name])

        data.setdefault('count', []).append(count)
        data.setdefault('missings', []).append(len(missings))
        data.setdefault('missings channels', []).append(missings)
        data.setdefault('montage', []).append(montage_name)

    df = pd.DataFrame.from_dict(data)
    return df.sort_values('count', ascending=False, ignore_index=True)





from .base import Database, BCIilliteracy, ALL
from typing import Optional, Tuple
import numpy as np
from . import databases
import warnings
import logging


########################################################################
class GIGA(Database):
    """"""
    metadata = databases.giga

    # ----------------------------------------------------------------------
    def load_subject(self, subject: int, mode: str = 'training') -> None:
        """"""
        data = super().load_subject(subject, mode)
        self.data = data['eeg'][0][0]

    # ----------------------------------------------------------------------
    def get_run(self, run: int, classes: Optional[list] = ALL, channels: Optional[list] = ALL, reject_bad_trials: Optional[bool] = True) -> Tuple[np.ndarray, np.ndarray]:
        """"""
        classes = self.format_class_selector(classes)
        channels = self.format_channels_selectors(channels)
        super().get_run(run, classes, channels, reject_bad_trials)

        # Index of starts of all cues
        all_cues = np.where(self.data[11][0] == 1)[0]

        # Split in runs and select the specified run
        trials_count = self.data[9][0][0]
        cues = np.array([all_cues[i:i + 20]
                         for i in range(0, trials_count, 20)][run])

        start = (self.metadata['sampling_rate'] * 2) - 1
        end = int(self.metadata['sampling_rate']
                  * self.metadata['duration']) + 1

        # reject bad trial
        if reject_bad_trials:
            bad_trials = []
            for cls in classes:
                trials_runs = np.ones((trials_count,), dtype=bool)
                # 14 bad trials--bad trials MI
                tmp = self.data[14][0][0][1][0][cls]
                if len(tmp) != 0:
                    trials_runs[tmp - 1] = 0
                bad_trials.extend([trials_runs[i:i + 20]
                                   for i in range(0, trials_count, 20)])
        #
        trials = []
        classes_out = []
        for cls in classes:
            data = self.data[7 + cls]  # classes starts in index 7
            if reject_bad_trials:
                # cls*max_runs -- run-1
                x = bad_trials[(cls * self.runs) + run]
                cues_r = cues[x]
                if len(cues_r):
                    trials.extend([data[:, cue - start:cue + end]
                                   for cue in cues_r])
                    classes_out.extend([cls] * len(cues_r))
            else:
                trials.extend([data[:, cue - start:cue + end] for cue in cues])
                classes_out.extend([cls] * len(cues))

        if not len(trials):
            logging.warning(
                f'The subject {self.subject} in the run {run} has no data.')
            return None, None

        # Select only EEG channels
        run = np.array(trials)[:, :len(self.metadata['channel_names']), :]

        # Select channels
        run = run[:, channels, :]

        return run, np.array(classes_out)

    # # ----------------------------------------------------------------------
    # def resting(self, channels: Optional[list] = ALL) -> np.ndarray:
        # """"""
        # channels = self.format_channels_selectors(channels)
        # resting = self.data[1]
        # resting = resting[channels]
        # return resting


########################################################################
class BCI2a(Database):
    """"""
    metadata = databases.bci2a

    # ----------------------------------------------------------------------
    def load_subject(self, subject: int, mode: str = 'training') -> None:
        """"""
        data = super().load_subject(subject, mode)
        self.data = data['data'][0]

    # ----------------------------------------------------------------------
    def get_run(self, run: int, classes: Optional[list] = ALL, channels: Optional[list] = ALL, reject_bad_trials: Optional[bool] = True) -> Tuple[np.ndarray, np.ndarray]:
        """"""
        classes = self.format_class_selector(classes)
        channels = self.format_channels_selectors(channels)
        super().get_run(run, classes, channels, reject_bad_trials)
        artifacts = self.data[3 + run][0][0][5].T[0] == 1

        # A04T contains only the eye movement condition
        if self.subject == 4 and self.mode == 'training':
            run = run - 2

        classes_list = np.array([i[0] for i in self.data[3 + run][0][0][2]])
        starts = [s[0] for s in self.data[3 + run][0][0][1]]
        end = int(self.metadata['sampling_rate'] * self.metadata['duration'])

        run = np.array([self.data[3 + run][0][0][0][start:start + end]
                        for start in starts])

        # Remove EOG
        run = run[:, :, :22]

        # Select channels
        run = run[:, :, channels]

        # trial x channel x time
        run = np.moveaxis(run, 2, 1)

        # reject bad trials
        if reject_bad_trials:
            run = run[~artifacts]
            classes_list = classes_list[~artifacts]

        idx = []
        c = []
        for cls in classes:
            idx.append(np.where(np.array(classes_list) == cls + 1)[0])
            c.append([cls] * len(idx[-1]))

        return run[np.concatenate(idx), :, :], np.concatenate(c)


########################################################################
class HighGamma(Database):
    """"""
    metadata = databases.highgamma

    # ----------------------------------------------------------------------
    def load_subject(self, subject: int, mode: str = 'training') -> None:
        """"""
        data = super().load_subject(subject, mode)

        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            self.data = data.root
            # DataTypeWarning: Unsupported type for attribute 'MATLAB_fields' in node 'mrk'
            self.data_mrk = data.root.mrk

    # ----------------------------------------------------------------------
    def get_run(self, run: int, classes: Optional[list] = ALL, channels: Optional[list] = ALL, reject_bad_trials: Optional[bool] = True) -> Tuple[np.ndarray, np.ndarray]:
        """"""
        classes = self.format_class_selector(classes)
        channels = self.format_channels_selectors(channels)
        super().get_run(run, classes, channels, reject_bad_trials)

        classes_list = self.data_mrk.event.desc.read()[0]
        starts = ((self.data_mrk.time.read() / 1000) * 500).T[0].astype(int)
        end = int(self.metadata['sampling_rate'] * self.metadata['duration'])

        data = np.concatenate(
            [getattr(self.data, f"ch{ch+1}") for ch in channels])

        run = np.array([data[:, start:start + end] for start in starts])

        idx = []
        c = []
        for cls in classes:
            idx.append(np.where(np.array(classes_list) == cls + 1)[0])
            c.append([cls] * len(idx[-1]))

        return run[np.concatenate(idx), :, :], np.concatenate(c)


########################################################################
class BCIilliteracy_MI(BCIilliteracy):
    """"""
    metadata = databases.bciilliteracy_mi

    # ----------------------------------------------------------------------
    def load_subject(self, subject: int, mode: str = 'training') -> None:
        """"""
        data_ = super().load_subject(subject, mode)

        if mode == 'training':
            self.data_ = [d['EEG_MI_train'][0][0] for d in data_]
        elif mode == 'evaluation':
            self.data_ = [d['EEG_MI_test'][0][0] for d in data_]


########################################################################
class BCIilliteracy_ERP(BCIilliteracy):
    """"""
    metadata = databases.bciilliteracy_erp

    # ----------------------------------------------------------------------
    def load_subject(self, subject: int, mode: str = 'training') -> None:
        """"""
        data_ = super().load_subject(subject, mode)

        if mode == 'training':
            self.data_ = [d['EEG_ERP_train'][0][0] for d in data_]
        elif mode == 'evaluation':
            self.data_ = [d['EEG_ERP_test'][0][0] for d in data_]


########################################################################
class BCIilliteracy_SSVEP(BCIilliteracy):
    """"""
    metadata = databases.bciilliteracy_ssvep

    # ----------------------------------------------------------------------
    def load_subject(self, subject: int, mode: str = 'training') -> None:
        """"""
        data_ = super().load_subject(subject, mode)

        if mode == 'training':
            self.data_ = [d['EEG_SSVEP_train'][0][0] for d in data_]
        elif mode == 'evaluation':
            self.data_ = [d['EEG_SSVEP_test'][0][0] for d in data_]



import os
import shutil
from jinja2 import Template
import logging
from colorama import Fore


TEMPLATE_EXTENSION = '.j2'


########################################################################
class Distutils:
    """"""

    # ----------------------------------------------------------------------
    def __init__(self, dst: str) -> None:
        """Constructor"""
        self.src = os.path.join(
            os.path.abspath(os.path.dirname(__file__)), 'template')
        self.dst = dst

        self.collect_data()
        self.clone_pkg()
        self.contextualize()

    # ----------------------------------------------------------------------
    def collect_data(self, **kwargs):
        """"""
        if not kwargs:
            print("""
This command line will help you to create a standard Python package with a
documentation template preconfigured and 'CGPDS' as the namespace.""", end="\n\n")

            print("Documentation: https://gcpds.readthedocs.io/projects/utils/en/latest/_notebooks/05-distutils.html", end="\n\n")

        def read(hint, default=None):
            if default:
                v = input(f"{Fore.MAGENTA}> {hint} [{default}]: {Fore.RESET}")
            else:
                v = input(f"{Fore.MAGENTA}> {hint}: {Fore.RESET}")

            if not v and not default:
                print(f"{Fore.RED}* Please enter some text.{Fore.RESET}")
                return read(hint)
            elif not v and default:
                return default
            else:
                return v

        pkg_name = read('Package name', default=kwargs.get('PKG_NAME', None))
        author = read('Author', default=kwargs.get('AUTHOR', None))
        author_email = read(
            'Author email', default=kwargs.get('AUTHOR_EMAIL', None))

        maintainer = read(
            f'Maintainer', default=kwargs.get('MAINTAINER', author))
        maintainer_email = read(
            f'Maintainer email', default=kwargs.get('MAINTAINER_EMAIL', author_email))

        requieres = read(f'Requieres (separated by comma)',
                         default=kwargs.get('PKG_REQUIERES', None))

        self.data = {
            'PKG_NAME': pkg_name.lower().replace(' ', '_').replace('-', '_'),
            'AUTHOR': author.title(),
            'AUTHOR_EMAIL': author_email.lower(),
            'MAINTAINER': maintainer.title(),
            'MAINTAINER_EMAIL': maintainer_email.lower(),
            'PKG_REQUIERES': ', '.join([p.strip() for p in requieres.split(',')]),
        }

        print('\n')
        for key in self.data:
            print(f"{Fore.GREEN}> {key}: {self.data[key]}{Fore.RESET}")

        ok = None
        while not ok in ['y', 'n']:
            ok = input(
                f'\n{Fore.MAGENTA}> input data is ok? [Y/n]: {Fore.RESET}')
            ok = ok.lower()
            ok = ok.strip()
            if ok == '':
                ok = 'y'

        if ok == 'n':
            return self.collect_data(**self.data)

    # ----------------------------------------------------------------------
    def clone_pkg(self):
        """"""

        self.pkg_user = os.path.join(self.dst, self.data['PKG_NAME'])

        try:
            shutil.copytree(self.src, self.pkg_user)
        except:
            logging.warning(
                f"Impossible to create the package '{os.path.split(self.dst)[1]}' "
                f"in '{os.path.split(self.dst)[0]}', already exists?")
            return

        os.rename(os.path.join(self.pkg_user, 'gcpds', '{{PKG_NAME}}'),
                  os.path.join(self.pkg_user, 'gcpds', self.data['PKG_NAME']))

    # ----------------------------------------------------------------------
    def contextualize(self):
        """"""
        for root, dirs, files in os.walk(self.pkg_user):

            for file in files:

                if not os.path.splitext(file)[-1] == TEMPLATE_EXTENSION:
                    continue

                filepath = os.path.join(root, file)

                with open(filepath, 'rb') as file:
                    content = file.read()

                template = Template(content.decode())
                content = template.render(**self.data)

                with open(filepath, 'wb') as file:
                    file.write(content.encode())

                os.rename(filepath, os.path.join(
                    *os.path.splitext(filepath)[:-1]))






import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
   README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='gcpds-utils',
    version='0.1a1',
    packages=['gcpds.utils'],

    author='Yeison Cardona',
    author_email='yencardonaal@unal.edu.co',
    maintainer='Yeison Cardona',
    maintainer_email='yencardonaal@unal.edu.co',

    download_url='',

    install_requires=['numpy',
                      'scipy',
                      'matplotlib',
                      'mne',
                      'tables',

                      'jinja2',
                      'colorama',
                      'sphinx',
                      'ipython',
                      'tqdm',

                      ],

    scripts=[
       "cmd/gcpds_distutils",
    ],


    include_package_data=True,
    license='Simplified BSD License',
    description="",
    zip_safe=False,

    long_description=README,
    long_description_content_type='text/markdown',

    python_requires='>=3.6',

    classifiers=[
       'Development Status :: 4 - Beta',
       'Intended Audience :: Developers',
       'Intended Audience :: Education',
       'Intended Audience :: Healthcare Industry',
       'Intended Audience :: Science/Research',
       'License :: OSI Approved :: BSD License',
       'Programming Language :: Python :: 3.7',
       'Programming Language :: Python :: 3.8',
       'Topic :: Scientific/Engineering',
       'Topic :: Scientific/Engineering :: Artificial Intelligence',
       'Topic :: Scientific/Engineering :: Human Machine Interfaces',
       'Topic :: Scientific/Engineering :: Medical Science Apps.',
       'Topic :: Software Development :: Embedded Systems',
       'Topic :: Software Development :: Libraries',
       'Topic :: Software Development :: Libraries :: Application Frameworks',
       'Topic :: Software Development :: Libraries :: Python Modules',
       'Topic :: System :: Hardware :: Hardware Drivers',
    ],

)



import numpy as np
from matplotlib import pyplot as plt
from matplotlib import cm
import matplotlib
import mne


# ----------------------------------------------------------------------
def plot_eeg(data, channels, fs, sca=1, cmap='viridis'):
    """"""
    plt.gcf()
    time = np.linspace(0, data.shape[1] / fs, data.shape[1])
    # sumf = sca * \
        # np.max(
            # data - np.matlib.repmat(data.mean(axis=1).reshape(-1, 1), 1, data.shape[1]))

    sumf = sca * np.max(data - data.mean(axis=1)[..., np.newaxis])

    cmap = cm.get_cmap(cmap, len(channels))
    [plt.plot(time, (ch - ch.mean()) + sumf * i, color=cmap(i), zorder=10)
     for i, ch in enumerate(data)]

    plt.yticks(np.arange(0, sumf * len(channels), sumf), channels)
    plt.grid(True, zorder=-1, alpha=0.2)
    plt.ylabel('Channels')
    plt.xlabel('Time [$s$]')


# ----------------------------------------------------------------------
def plot_topoplot(data, channels, fs, sca=1, cmap='plasma', montage='standard_1020', **kwargs):

    info = mne.create_info(channels, sfreq=fs, ch_types="eeg")
    info.set_montage(montage)

    fig = plt.gcf()
    ax = plt.gca()

#     Colorbar
    cax = fig.add_axes([0.95, 0.15, 0.05, 0.75])
    norm = matplotlib.colors.Normalize(vmin=kwargs.get(
        'vmin', data.min()), vmax=kwargs.get('vmax', data.max()))
    sm = cm.ScalarMappable(cmap=cmap, norm=norm)
    sm.set_array([])
    plt.colorbar(sm, cax=cax)

    mne.viz.plot_topomap(data, info, names=channels,
                         cmap=cmap, axes=ax, **kwargs)




from google.colab import drive
import os
import sys
import logging

SHARED_DRIVE = 'GCPDS'


# ----------------------------------------------------------------------
def mount(shared_drive=SHARED_DRIVE, mount='drive', force_remount=False):
    """"""
    mount_dst = os.path.join('/', 'content', mount)
    drive.mount(mount_dst, force_remount=force_remount)

    gcpds_drive = os.path.join(mount_dst, 'Shared drives', shared_drive)
    if not os.path.exists(gcpds_drive):
        logging.warning(
            f"You haven't access to {shared_drive} shared drive, ask for one.")
        return

    os.chdir(gcpds_drive)
    logging.warning(
        f"Your working directory is now the root of the shared drive '{shared_drive}'.")



# BCI2a, GIGA_Laplacian, databases
from .loaddb import GIGA, BCI2a, HighGamma, BCIilliteracy_MI, BCIilliteracy_ERP, BCIilliteracy_SSVEP

available_databases = 'GIGA BCI2a HighGamma BCIilliteracy_MI BCIilliteracy_ERP BCIilliteracy_SSVEP'.split()



import json
import os

fids = json.load(
    open(os.path.join(os.path.dirname(__file__), 'fids.json'), 'rb'))


giga = {
    'channel_names': ['Fp1', 'AF7', 'AF3', 'F1', 'F3', 'F5', 'F7', 'FT7', 'FC5', 'FC3', 'FC1', 'C1', 'C3', 'C5', 'T7', 'TP7', 'CP5', 'CP3', 'CP1', 'P1', 'P3', 'P5', 'P7', 'P9', 'PO7', 'PO3', 'O1', 'Iz', 'Oz', 'POz', 'Pz', 'CPz', 'Fpz', 'Fp2', 'AF8', 'AF4', 'AFz', 'Fz', 'F2', 'F4', 'F6', 'F8', 'FT8', 'FC6', 'FC4', 'FC2', 'FCz', 'Cz', 'C2', 'C4', 'C6', 'T8', 'TP8', 'CP6', 'CP4', 'CP2', 'P2', 'P4', 'P6', 'P8', 'P10', 'PO8', 'PO4', 'O2'],
    'classes': ['left hand', 'right hand'],
    'sampling_rate': 512,
    'montage': 'standard_1005',
    'tmin': -2,
    'duration': 5,
    'reference': '',
    'runs_training': [5, 5, 5, 5, 5, 5, 6, 5, 6, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 6, 5, 5, 5, 5, 5, 5],
    'runs_evaluation': [],

    'subject_training_files': fids['GIGA training'],
    'subject_training_pattern': lambda subject: f's{str(subject).rjust(2, "0")}.mat',

    'subject_evaluation_files': {},
    'subject_training_pattern': lambda subject: f's{str(subject).rjust(2, "0")}.mat',

    'metadata': fids['GIGA metadata'],
}


bci2a = {
    'channel_names': ['Fz', 'FC3', 'FC1', 'FCz', 'FC2', 'FC4', 'C5', 'C3', 'C1', 'Cz', 'C2', 'C4', 'C6', 'CP3', 'CP1', 'CPz', 'CP2', 'CP4', 'P1', 'Pz', 'P2', 'POz'],
    'classes': ['left hand', 'right hand', 'feet', 'tongue'],
    'sampling_rate': 250,
    'montage': 'standard_1020',
    'tmin': -2,
    'duration': 7,
    'reference': '',
    'runs_training': [6, 6, 6, 6, 6, 6, 6, 6, 6],
    'runs_evaluation': [6, 6, 6, 6, 6, 6, 6, 6, 6],

    'subject_training_files': fids['BCI2a training'],
    'subject_training_pattern': lambda subject: f'A{str(subject).rjust(2, "0")}T.mat',

    'subject_evaluation_files': fids['BCI2a evaluation'],
    'subject_evaluation_pattern': lambda subject: f'A{str(subject).rjust(2, "0")}E.mat',

    'metadata': fids['BCI2a metadata'],

}


highgamma = {
    'channel_names': ['Fp1', 'Fp2', 'Fpz', 'F7', 'F3', 'Fz', 'F4', 'F8', 'FC5', 'FC1', 'FC2', 'FC6', 'M1', 'T7', 'C3', 'Cz', 'C4', 'T8', 'M2', 'CP5', 'CP1', 'CP2', 'CP6', 'P7', 'P3', 'Pz', 'P4', 'P8', 'POz', 'O1', 'Oz', 'O2', 'EOGh', 'EOGv', 'EMG_RH', 'EMG_LH', 'EMG_RF', 'AF7', 'AF3', 'AF4', 'AF8', 'F5', 'F1', 'F2', 'F6', 'FC3', 'FCz', 'FC4', 'C5', 'C1', 'C2', 'C6', 'CP3', 'CPz', 'CP4', 'P5', 'P1', 'P2', 'P6', 'PO5', 'PO3', 'PO4', 'PO6', 'FT7', 'FT8', 'TP7', 'TP8', 'PO7', 'PO8', 'FT9', 'FT10', 'TPP9h', 'TPP10h', 'PO9', 'PO10', 'P9', 'P10', 'AFF1', 'AFz', 'AFF2', 'FFC5h', 'FFC3h', 'FFC4h', 'FFC6h', 'FCC5h', 'FCC3h', 'FCC4h', 'FCC6h', 'CCP5h', 'CCP3h', 'CCP4h', 'CCP6h', 'CPP5h', 'CPP3h', 'CPP4h', 'CPP6h', 'PPO1', 'PPO2', 'I1', 'Iz', 'I2', 'AFp3h', 'AFp4h', 'AFF5h', 'AFF6h', 'FFT7h', 'FFC1h', 'FFC2h', 'FFT8h', 'FTT9h', 'FTT7h', 'FCC1h', 'FCC2h', 'FTT8h', 'FTT10h', 'TTP7h', 'CCP1h', 'CCP2h', 'TTP8h', 'TPP7h', 'CPP1h', 'CPP2h', 'TPP8h', 'PPO9h', 'PPO5h', 'PPO6h', 'PPO10h', 'POO9h', 'POO3h', 'POO4h', 'POO10h', 'OI1h', 'OI2h'],
    'classes': ['right hand', 'left hand', 'rest', 'feet'],
    'sampling_rate': 500,
    'montage': 'standard_1005',
    'tmin': 0,
    'duration': 4,
    'reference': '',
    'runs_training': [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
    'runs_evaluation': [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],

    'subject_training_files': fids['HighGamma training'],
    'subject_training_pattern': lambda subject: os.path.join('train', f'{subject}.mat'),

    'subject_evaluation_files': fids['HighGamma evaluation'],
    'subject_evaluation_pattern': lambda subject: os.path.join('test', f'{subject}.mat'),

    'metadata': fids['HighGamma metadata'],
}


bciilliteracy_mi = {
    'channel_names': ['Fp1', 'Fp2', 'F7', 'F3', 'Fz', 'F4', 'F8', 'FC5', 'FC1', 'FC2', 'FC6', 'T7', 'C3', 'Cz', 'C4', 'T8', 'TP9', 'CP5', 'CP1', 'CP2', 'CP6', 'TP10', 'P7', 'P3', 'Pz', 'P4', 'P8', 'PO9', 'O1', 'Oz', 'O2', 'PO10', 'FC3', 'FC4', 'C5', 'C1', 'C2', 'C6', 'CP3', 'CPz', 'CP4', 'P1', 'P2', 'POz', 'FT9', 'FTT9h', 'TTP7h', 'TP7', 'TPP9h', 'FT10', 'FTT10h', 'TPP8h', 'TP8', 'TPP10h', 'F9', 'F10', 'AF7', 'AF3', 'AF4', 'AF8', 'PO3', 'PO4'],
    'classes': ['right', 'left'],
    'sampling_rate': 1000,
    'montage': 'standard_1020',
    'tmin': -3,
    'duration': 7,
    'reference': '',
    'runs': [2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2],

    'subject_files': list(filter(lambda f: 'MI' in f, fids['BCIilliteracy'])),
    'subject_pattern': lambda subject, run: os.path.join(f'session{run}', f'sess{str(run).rjust(2, "0")}_subj{str(subject).rjust(2, "0")}_EEG_MI.mat'),

    'metadata': fids['BCIilliteracy metadata'],

}


bciilliteracy_erp = bciilliteracy_mi.copy()
bciilliteracy_erp.update({
    'classes': ['target', 'nontarget'],
    'tmin': -4.5,
    'duration': 14.5,
    'subject_files': list(filter(lambda f: 'ERP' in f, fids['BCIilliteracy'])),
    'subject_pattern': lambda subject, run: os.path.join(f'session{run}', f'sess{str(run).rjust(2, "0")}_subj{str(subject).rjust(2, "0")}_EEG_ERP.mat'),
})


bciilliteracy_ssvep = bciilliteracy_mi.copy()
bciilliteracy_ssvep.update({
    'classes': ['up', 'left', 'right', 'down'],
    'tmin': -4,
    'duration': 8,
    'subject_files': list(filter(lambda f: 'SSVEP' in f, fids['BCIilliteracy'])),
    'subject_pattern': lambda subject, run: os.path.join(f'session{run}', f'sess{str(run).rjust(2, "0")}_subj{str(subject).rjust(2, "0")}_EEG_SSVEP.mat'),
})
